@@ -28,6 +28,14 @@
 
 This AWS Solutions Construct implements the AWS Lambda function and Amazon Elasticsearch Service with the least privileged permissions.
 
+**Some cluster configurations (e.g VPC access) require the existence of the `AWSServiceRoleForAmazonElasticsearchService` Service-Linked Role in your account.**
+
+**You will need to create the service-linked role using the AWS CLI once in any account using this construct (it may have already been run to support other stacks):**
+
+```
+aws iam create-service-linked-role --aws-service-name es.amazonaws.com
+```
+
 Here is a minimal deployable pattern definition:
 
 Typescript
@@ -112,7 +120,10 @@ new LambdaToElasticSearchAndKibana(this, "sample",
 |domainName|`string`|Domain name for the Cognito and the Elasticsearch Service|
 |cognitoDomainName?|`string`|Optional Cognito Domain Name, if provided it will be used for Cognito Domain, and domainName will be used for the Elasticsearch Domain|
 |createCloudWatchAlarms|`boolean`|Whether to create recommended CloudWatch alarms|
-|domainEndpointEnvironmentVariableName?|`string`|Optional Name for the Lambda function environment variable set to the domain endpoint. Default: DOMAIN_ENDPOINT |
+|domainEndpointEnvironmentVariableName?|`string`|Optional Name for the ElasticSearch domain endpoint environment variable set for the Lambda function.|
+|existingVpc?|[`ec2.IVpc`](https://docs.aws.amazon.com/cdk/api/latest/docs/@aws-cdk_aws-ec2.IVpc.html)|An optional, existing VPC into which this pattern should be deployed. When deployed in a VPC, the Lambda function will use ENIs in the VPC to access network resources. If an existing VPC is provided, the `deployVpc` property cannot be `true`. This uses `ec2.IVpc` to allow clients to supply VPCs that exist outside the stack using the [`ec2.Vpc.fromLookup()`](https://docs.aws.amazon.com/cdk/api/latest/docs/@aws-cdk_aws-ec2.Vpc.html#static-fromwbrlookupscope-id-options) method.|
+|vpcProps?|[`ec2.VpcProps`](https://docs.aws.amazon.com/cdk/api/latest/docs/@aws-cdk_aws-ec2.VpcProps.html)|Optional user provided properties to override the default properties for the new VPC. `enableDnsHostnames`, `enableDnsSupport`, `natGateways` and `subnetConfiguration` are set by the pattern, so any values for those properties supplied here will be overrriden. If `deployVpc` is not `true` then this property will be ignored.|
+|deployVpc?|`boolean`|Whether to create a new VPC based on `vpcProps` into which to deploy this pattern. Setting this to true will deploy the minimal, most private VPC to run the pattern:<ul><li> One isolated subnet in each Availability Zone used by the CDK program</li><li>`enableDnsHostnames` and `enableDnsSupport` will both be set to true</li></ul>If this property is `true` then `existingVpc` cannot be specified. Defaults to `false`.|
 
 ## Pattern Properties
 
@@ -125,6 +136,7 @@ new LambdaToElasticSearchAndKibana(this, "sample",
 |elasticsearchDomain|[`elasticsearch.CfnDomain`](https://docs.aws.amazon.com/cdk/api/latest/docs/@aws-cdk_aws-elasticsearch.CfnDomain.html)|Returns an instance of elasticsearch.CfnDomain created by the construct|
 |elasticsearchDomain|[`iam.Role`](https://docs.aws.amazon.com/cdk/api/latest/docs/@aws-cdk_aws-iam.Role.html)|Returns an instance of iam.Role created by the construct for elasticsearch.CfnDomain|
 |cloudwatchAlarms?|[`cloudwatch.Alarm[]`](https://docs.aws.amazon.com/cdk/api/latest/docs/@aws-cdk_aws-cloudwatch.Alarm.html)|Returns a list of cloudwatch.Alarm created by the construct|
+|vpc?|[`ec2.IVpc`](https://docs.aws.amazon.com/cdk/api/latest/docs/@aws-cdk_aws-ec2.IVpc.html)|Returns an interface on the VPC used by the pattern (if any). This may be a VPC created by the pattern or the VPC supplied to the pattern constructor.|
 
 ## Lambda Function
 
@@ -180,6 +192,7 @@ from ._jsii import *
 
 import aws_cdk.aws_cloudwatch
 import aws_cdk.aws_cognito
+import aws_cdk.aws_ec2
 import aws_cdk.aws_elasticsearch
 import aws_cdk.aws_iam
 import aws_cdk.aws_lambda
@@ -199,10 +212,13 @@ class LambdaToElasticSearchAndKibana(
         domain_name: builtins.str,
         cognito_domain_name: typing.Optional[builtins.str] = None,
         create_cloud_watch_alarms: typing.Optional[builtins.bool] = None,
+        deploy_vpc: typing.Optional[builtins.bool] = None,
         domain_endpoint_environment_variable_name: typing.Optional[builtins.str] = None,
         es_domain_props: typing.Optional[aws_cdk.aws_elasticsearch.CfnDomainProps] = None,
         existing_lambda_obj: typing.Optional[aws_cdk.aws_lambda.Function] = None,
+        existing_vpc: typing.Optional[aws_cdk.aws_ec2.IVpc] = None,
         lambda_function_props: typing.Optional[aws_cdk.aws_lambda.FunctionProps] = None,
+        vpc_props: typing.Optional[aws_cdk.aws_ec2.VpcProps] = None,
     ) -> None:
         '''
         :param scope: - represents the scope for all the resources.
@@ -210,10 +226,13 @@ class LambdaToElasticSearchAndKibana(
         :param domain_name: Cognito & ES Domain Name. Default: - None
         :param cognito_domain_name: Optional Cognito Domain Name, if provided it will be used for Cognito Domain, and domainName will be used for the Elasticsearch Domain. Default: - None
         :param create_cloud_watch_alarms: Whether to create recommended CloudWatch alarms. Default: - Alarms are created
+        :param deploy_vpc: Whether to deploy a new VPC. Default: - false
         :param domain_endpoint_environment_variable_name: Optional Name for the Lambda function environment variable set to the domain endpoint. Default: - DOMAIN_ENDPOINT
         :param es_domain_props: Optional user provided props to override the default props for the Elasticsearch Service. Default: - Default props are used
         :param existing_lambda_obj: Existing instance of Lambda Function object, providing both this and ``lambdaFunctionProps`` will cause an error. Default: - None
+        :param existing_vpc: An existing VPC for the construct to use (construct will NOT create a new VPC in this case). Default: - None
         :param lambda_function_props: User provided props to override the default props for the Lambda function. Default: - Default props are used
+        :param vpc_props: Properties to override default properties if deployVpc is true. Default: - DefaultIsolatedVpcProps() in vpc-defaults.ts
 
         :access: public
         :since: 0.8.0
@@ -223,14 +242,29 @@ class LambdaToElasticSearchAndKibana(
             domain_name=domain_name,
             cognito_domain_name=cognito_domain_name,
             create_cloud_watch_alarms=create_cloud_watch_alarms,
+            deploy_vpc=deploy_vpc,
             domain_endpoint_environment_variable_name=domain_endpoint_environment_variable_name,
             es_domain_props=es_domain_props,
             existing_lambda_obj=existing_lambda_obj,
+            existing_vpc=existing_vpc,
             lambda_function_props=lambda_function_props,
+            vpc_props=vpc_props,
         )
 
         jsii.create(self.__class__, self, [scope, id, props])
 
+    @jsii.member(jsii_name="setupCognito")
+    def setup_cognito(
+        self,
+        scope: aws_cdk.core.Construct,
+        domain_name: builtins.str,
+    ) -> typing.Mapping[typing.Any, typing.Any]:
+        '''
+        :param scope: -
+        :param domain_name: -
+        '''
+        return typing.cast(typing.Mapping[typing.Any, typing.Any], jsii.invoke(self, "setupCognito", [scope, domain_name]))
+
     @builtins.property # type: ignore[misc]
     @jsii.member(jsii_name="elasticsearchDomain")
     def elasticsearch_domain(self) -> aws_cdk.aws_elasticsearch.CfnDomain:
@@ -268,6 +302,11 @@ class LambdaToElasticSearchAndKibana(
     ) -> typing.Optional[typing.List[aws_cdk.aws_cloudwatch.Alarm]]:
         return typing.cast(typing.Optional[typing.List[aws_cdk.aws_cloudwatch.Alarm]], jsii.get(self, "cloudwatchAlarms"))
 
+    @builtins.property # type: ignore[misc]
+    @jsii.member(jsii_name="vpc")
+    def vpc(self) -> typing.Optional[aws_cdk.aws_ec2.IVpc]:
+        return typing.cast(typing.Optional[aws_cdk.aws_ec2.IVpc], jsii.get(self, "vpc"))
+
 
 @jsii.data_type(
     jsii_type="@aws-solutions-constructs/aws-lambda-elasticsearch-kibana.LambdaToElasticSearchAndKibanaProps",
@@ -276,10 +315,13 @@ class LambdaToElasticSearchAndKibana(
         "domain_name": "domainName",
         "cognito_domain_name": "cognitoDomainName",
         "create_cloud_watch_alarms": "createCloudWatchAlarms",
+        "deploy_vpc": "deployVpc",
         "domain_endpoint_environment_variable_name": "domainEndpointEnvironmentVariableName",
         "es_domain_props": "esDomainProps",
         "existing_lambda_obj": "existingLambdaObj",
+        "existing_vpc": "existingVpc",
         "lambda_function_props": "lambdaFunctionProps",
+        "vpc_props": "vpcProps",
     },
 )
 class LambdaToElasticSearchAndKibanaProps:
@@ -289,19 +331,25 @@ class LambdaToElasticSearchAndKibanaProps:
         domain_name: builtins.str,
         cognito_domain_name: typing.Optional[builtins.str] = None,
         create_cloud_watch_alarms: typing.Optional[builtins.bool] = None,
+        deploy_vpc: typing.Optional[builtins.bool] = None,
         domain_endpoint_environment_variable_name: typing.Optional[builtins.str] = None,
         es_domain_props: typing.Optional[aws_cdk.aws_elasticsearch.CfnDomainProps] = None,
         existing_lambda_obj: typing.Optional[aws_cdk.aws_lambda.Function] = None,
+        existing_vpc: typing.Optional[aws_cdk.aws_ec2.IVpc] = None,
         lambda_function_props: typing.Optional[aws_cdk.aws_lambda.FunctionProps] = None,
+        vpc_props: typing.Optional[aws_cdk.aws_ec2.VpcProps] = None,
     ) -> None:
         '''
         :param domain_name: Cognito & ES Domain Name. Default: - None
         :param cognito_domain_name: Optional Cognito Domain Name, if provided it will be used for Cognito Domain, and domainName will be used for the Elasticsearch Domain. Default: - None
         :param create_cloud_watch_alarms: Whether to create recommended CloudWatch alarms. Default: - Alarms are created
+        :param deploy_vpc: Whether to deploy a new VPC. Default: - false
         :param domain_endpoint_environment_variable_name: Optional Name for the Lambda function environment variable set to the domain endpoint. Default: - DOMAIN_ENDPOINT
         :param es_domain_props: Optional user provided props to override the default props for the Elasticsearch Service. Default: - Default props are used
         :param existing_lambda_obj: Existing instance of Lambda Function object, providing both this and ``lambdaFunctionProps`` will cause an error. Default: - None
+        :param existing_vpc: An existing VPC for the construct to use (construct will NOT create a new VPC in this case). Default: - None
         :param lambda_function_props: User provided props to override the default props for the Lambda function. Default: - Default props are used
+        :param vpc_props: Properties to override default properties if deployVpc is true. Default: - DefaultIsolatedVpcProps() in vpc-defaults.ts
 
         :summary: The properties for the CognitoToApiGatewayToLambda Construct
         '''
@@ -309,6 +357,8 @@ class LambdaToElasticSearchAndKibanaProps:
             es_domain_props = aws_cdk.aws_elasticsearch.CfnDomainProps(**es_domain_props)
         if isinstance(lambda_function_props, dict):
             lambda_function_props = aws_cdk.aws_lambda.FunctionProps(**lambda_function_props)
+        if isinstance(vpc_props, dict):
+            vpc_props = aws_cdk.aws_ec2.VpcProps(**vpc_props)
         self._values: typing.Dict[str, typing.Any] = {
             "domain_name": domain_name,
         }
@@ -316,14 +366,20 @@ class LambdaToElasticSearchAndKibanaProps:
             self._values["cognito_domain_name"] = cognito_domain_name
         if create_cloud_watch_alarms is not None:
             self._values["create_cloud_watch_alarms"] = create_cloud_watch_alarms
+        if deploy_vpc is not None:
+            self._values["deploy_vpc"] = deploy_vpc
         if domain_endpoint_environment_variable_name is not None:
             self._values["domain_endpoint_environment_variable_name"] = domain_endpoint_environment_variable_name
         if es_domain_props is not None:
             self._values["es_domain_props"] = es_domain_props
         if existing_lambda_obj is not None:
             self._values["existing_lambda_obj"] = existing_lambda_obj
+        if existing_vpc is not None:
+            self._values["existing_vpc"] = existing_vpc
         if lambda_function_props is not None:
             self._values["lambda_function_props"] = lambda_function_props
+        if vpc_props is not None:
+            self._values["vpc_props"] = vpc_props
 
     @builtins.property
     def domain_name(self) -> builtins.str:
@@ -353,6 +409,15 @@ class LambdaToElasticSearchAndKibanaProps:
         result = self._values.get("create_cloud_watch_alarms")
         return typing.cast(typing.Optional[builtins.bool], result)
 
+    @builtins.property
+    def deploy_vpc(self) -> typing.Optional[builtins.bool]:
+        '''Whether to deploy a new VPC.
+
+        :default: - false
+        '''
+        result = self._values.get("deploy_vpc")
+        return typing.cast(typing.Optional[builtins.bool], result)
+
     @builtins.property
     def domain_endpoint_environment_variable_name(
         self,
@@ -384,6 +449,15 @@ class LambdaToElasticSearchAndKibanaProps:
         result = self._values.get("existing_lambda_obj")
         return typing.cast(typing.Optional[aws_cdk.aws_lambda.Function], result)
 
+    @builtins.property
+    def existing_vpc(self) -> typing.Optional[aws_cdk.aws_ec2.IVpc]:
+        '''An existing VPC for the construct to use (construct will NOT create a new VPC in this case).
+
+        :default: - None
+        '''
+        result = self._values.get("existing_vpc")
+        return typing.cast(typing.Optional[aws_cdk.aws_ec2.IVpc], result)
+
     @builtins.property
     def lambda_function_props(
         self,
@@ -395,6 +469,15 @@ class LambdaToElasticSearchAndKibanaProps:
         result = self._values.get("lambda_function_props")
         return typing.cast(typing.Optional[aws_cdk.aws_lambda.FunctionProps], result)
 
+    @builtins.property
+    def vpc_props(self) -> typing.Optional[aws_cdk.aws_ec2.VpcProps]:
+        '''Properties to override default properties if deployVpc is true.
+
+        :default: - DefaultIsolatedVpcProps() in vpc-defaults.ts
+        '''
+        result = self._values.get("vpc_props")
+        return typing.cast(typing.Optional[aws_cdk.aws_ec2.VpcProps], result)
+
     def __eq__(self, rhs: typing.Any) -> builtins.bool:
         return isinstance(rhs, self.__class__) and rhs._values == self._values
 
